from plone.indexer import indexer
from plone.multilingual.interfaces import ITranslatable, ITG, ILanguage


@indexer(ITranslatable)
def itgIndexer(obj):
    return ITG(obj, None)


@indexer(ITranslatable)
def LanguageIndexer(object, **kw):
    language = ILanguage(object).get_language()
    return language


import unittest2 as unittest

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles

from ..interfaces import ILanguage
from ..testing import PLONEMULTILINGUAL_INTEGRATION_TESTING

import transaction


class TestSubscribers(unittest.TestCase):
    """There are some events that are fired when an object
    is created, moved or copied.

    plone.multilingual registers some subscribres for each event
    to change the language of the object from the container where
    it has been created, moved or copied
    """
    layer = PLONEMULTILINGUAL_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

        self.folder_it = self._add_content(self.portal, 'Folder', 'it')
        self.folder_en = self._add_content(self.portal, 'Folder', 'en')
        ILanguage(self.folder_it).set_language('it')

    def _add_content(self, container, ptype, id_):
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        content_id = container.invokeFactory(ptype, id_)
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        return container[content_id]

    def test_created_event(self):
        """when an object is created in a folder
        it takes its language from the folder itself
        """
        folder_1 = self._add_content(self.folder_it, 'Folder', 'folder_1')
        self.assertEqual(ILanguage(folder_1).get_language(), 'it')

    def test_moved_event(self):
        folder = self._add_content(self.folder_it, 'Folder', 'folder_1')
        transaction.commit()
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        id_ = self.folder_it.manage_cutObjects(folder.getId())
        self.folder_en.manage_pasteObjects(id_)
        copied_folder = getattr(self.folder_en, "folder_1")
        self.assertEqual(ILanguage(copied_folder).get_language(), 'en')

    def test_copied_event(self):
        folder = self._add_content(self.folder_it, 'Folder', 'folder_1')
        id_ = self.folder_it.manage_copyObjects(folder.getId())
        self.folder_en.manage_pasteObjects(id_)
        copied_folder = getattr(self.folder_en, "folder_1")
        self.assertEqual(ILanguage(copied_folder).get_language(), 'en')



from Acquisition import aq_parent
from Products.CMFPlone.interfaces import IPloneSiteRoot
from plone.multilingual.interfaces import (
    LANGUAGE_INDEPENDENT,
    ITranslationManager,
    ITranslatable,
    ILanguage)
from Products.CMFCore.utils import getToolByName
from zope.component.hooks import getSite
from zope.lifecycleevent import modified, ObjectCopiedEvent
from Products.CMFCore.interfaces import IFolderish

def remove_translation_on_delete(obj, event):
    pass

# In case is a normal content update translation manager with the new language
def update_on_modify(obj, event):
    ITranslationManager(obj).update()


def set_recursive_language(obj, language):
    """ Set the language at this object and recursive
    """
    ILanguage(obj).set_language(language)
    modified(obj)
    if IFolderish.providedBy(obj):
        for item in obj.items():
            if ITranslatable.providedBy(item):
                set_recursive_language(item, language)


# Subscriber to set language on the child folder
def createdEvent(obj, event):
    portal = getSite()
    if isinstance(event, ObjectCopiedEvent):
        parent = aq_parent(event.object)
    else:
        parent = event.newParent
    language_tool = getToolByName(portal, 'portal_languages')
    if (language_tool.startNeutral() and
        ITranslatable.providedBy(obj)):
        # We leave this untouched by now.
        set_recursive_language(obj, LANGUAGE_INDEPENDENT)
    elif (IPloneSiteRoot.providedBy(parent) and
          ITranslatable.providedBy(obj) and
          ILanguage(obj).get_language() == LANGUAGE_INDEPENDENT):
        pl = getToolByName(portal, "portal_languages")
        language = pl.getPreferredLanguage()
        set_recursive_language(obj, language)
    elif ITranslatable.providedBy(parent):
        language = ILanguage(parent).get_language()
        set_recursive_language(obj, language)



from zope.interface import Interface, Attribute

# CONSTANTS
LANGUAGE_INDEPENDENT = ''

ATTRIBUTE_NAME = '_plone.tg'

NOTG = 'notg'

# Language-support
class ILanguage(Interface):

    def get_language(self):
        """ return the contents language """

    def set_language(self):
        """ return the contents language """


# Marker interface
class ITranslatable(Interface):
    """Marker interface for content types that support translation"""


# adapters
class ITranslationFactory(Interface):
    """Adapts ITranslated and is capable of returning
       a translation clone to be added.
    """

    def __call__(language):
        """Create a clone of the context
           for translation to the given language
        """


class ITranslationLocator(Interface):
    """Find a parent folder for a translation.
       Adapts ITranslated.
    """

    def __call__(language):
        """Return a parent folder into which a new translation can be added"""


class ITranslationIdChooser(Interface):
    """Find a valid id for a translation
       Adapts ITranslated.
    """

    def __call__(parent, language):
        """ Return a valid id for the translation """


class ITranslationCloner(Interface):
    """Subscription adapters to perform various aspects of cloning an object.
       Allows componentisation of things like workflow history cloning.
       Adapts ITranslated.
    """

    def __call__(object):
        """Update the translation copy that is being constructed"""



class ITranslationManager(Interface):

    def add_translation(object, intid):
        """
        create the translated content and register the translation
        """

    def remove_translation(language):
        """
        remove translation if exists (unregister the translation)
        """

    def get_translation(language):
        """
        get translation (translated object) if exists
        """

    def get_translations():
        """
        get all the translated objects (including the context)
        """

    def get_translated_languages():
        """
        get a list of the translated languages
        (language-code like 'en', 'it' etc. )
        """

    def register_translation(language, content):
        """
        register an existing content as translation
        for context
        """

    def update():
        """
        update the item registered in the canonical
        (used for changing the contexts language)
        """

    def query_canonical():
        """
        query if there is an canonical for the context
        used for migration
        """




class ILanguageIndependentFieldsManager(Interface):
    context = Attribute("context", "A translatable object")

    def copy_fields(translation):
        """ Copy language independent fields to translation."""


class IMutableTG(Interface):
    """Adapt an object to this interface to manage the TG of an object
    
    Be sure of what you are doing. TG is supposed to be stable and
    widely used
    """
    def get():
        """Return the TG of the context"""
    
    def set(tg):
        """Set the unique id of the context with the tg value.
        """


class ITG(Interface):
    """Abstract representation of a TG.

    Adapt an object to this interface to obtain its UUID. Adaptation will
    fail if the object does not have a TG (yet).
    """


# XXX For Upgrade step to 03 , Removing on the future



class ICanonical(Interface):

    languages = Attribute(u"dictionary {LANG_KEY: UUID, ...}")

    def add_item(language, intid):
        """ """

    def remove_item(language):
        """ """

    def get_item(language):
        """ """

    def remove_item_by_language(language):
        """ """

    def remove_item_by_id(id):
        """ """

    def get_items():
        """ """

    def get_keys():
        """ """


class IMultilingualStorage(Interface):
    """ Stores the canonical objects at the portal_multilingual tool """

    def add_canonical(id, canonical):
        """ add canonical """

    def get_canonical(id):
        """ get canonical """

    def remove_canonical(id):
        """ remove canonical """

    def get_canonicals():
        """ get all canonicals """


class ICanonicalStorage(Interface):
    """ Deprecated in 0.2, for migration purposes only """

    def add_canonical(id, canonical):
        """ add canonical """

    def get_canonical(id):
        """ get canonical """

    def remove_canonical(id):
        """ remove canonical """

# XXX




from Products.CMFCore.utils import getToolByName
from plone.dexterity.interfaces import IDexterityContent
from plone.uuid.handlers import addAttributeUUID
from plone.uuid.interfaces import IUUID
from zope.component import getUtility, queryUtility
from zope.component import getAllUtilitiesRegisteredFor
from zope.intid.interfaces import IIntIds

from plone.multilingual.canonical import Canonical

from plone.multilingual.interfaces import IMultilingualStorage

from Products.Five.browser import BrowserView
import logging


def upgrade(context):
    # Add missing UIDs to already existing dexterity content types
    # Borrowed from plone.app.dexterity :)

    logger = logging.getLogger('plone.multilingual')
    logger.info('Adding missing UUIDs to the already existing '
                'dexterity content types')

    catalog = getToolByName(context, 'portal_catalog')
    query = {'object_provides': IDexterityContent.__identifier__}
    results = catalog.unrestrictedSearchResults(query)
    for b in results:
        ob = b.getObject()
        if IUUID(ob, None) is None:
            addAttributeUUID(ob, None)
            ob.reindexObject(idxs=['UID'])
    logger.info('Added %s missing UUIDs' % len(results))

    # Upgrade utility, and convert intids to uuids
    intids = queryUtility(IIntIds)
    if not intids:
        intids = getAllUtilitiesRegisteredFor(IIntIds)[0]

    oldstorage = getToolByName(context, 'canonical_storage')

    for canonicalintid in oldstorage.canonicals:
        canonicaluuid = IUUID(intids.getObject(canonicalintid))
        translations = oldstorage.canonicals[canonicalintid].languages
        upgradedcanonical = Canonical()
        for lang in translations.keys():
            langintid = translations[lang]
            langobj = intids.getObject(langintid)
            languuid = IUUID(langobj)
            upgradedcanonical.add_item(lang, languuid)
        storage = getUtility(IMultilingualStorage)
        storage.add_canonical(canonicaluuid, upgradedcanonical)
        logger.info('%s' % upgradedcanonical.languages)


class upgradeView(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        upgrade(self.context)
        return 'plone.multilingual utility upgraded successfully'



from zope import interface
from zope.event import notify
from plone.uuid.interfaces import IUUID
from plone.multilingual.interfaces import (
    ILanguage,
    ITranslationManager,
    ITranslationFactory,
    ITG,
    IMutableTG,
    NOTG
)
from plone.multilingual.handlers import addAttributeTG
from plone.multilingual.events import (
    ObjectWillBeTranslatedEvent,
    ObjectTranslatedEvent,
)
from plone.uuid.handlers import addAttributeUUID
from plone.app.uuid.utils import uuidToObject
from zope.site.hooks import getSite
from Products.CMFCore.utils import getToolByName


class TranslationManager(object):

    interface.implements(ITranslationManager)

    def __init__(self, context):
        self.context = context
        if isinstance(context, str):
            self.tg = context
        else:
            self.tg = self.get_tg(context)
        self._canonical = None
        site = getSite()
        self.pcatalog = getToolByName(site, 'portal_catalog', None)

    def get_id(self, context):
        """If an object is created via portal factory we don't get a id, we
           have to wait till the object is really created.
           TODO: a better check if we are in the portal factory!
        """
        try:
            context_id = IUUID(context)
        # We must ensure that this case can't happen, any object translatable
        # will have an UUID (in any case we can be at the portal factory!)
        except KeyError:
            addAttributeUUID(context, None)
            context.reindexObject(idxs=['UID'])
            context_id = IUUID(context)
        return context_id

    def get_tg(self, context):
        """If an object is created via portal factory we don't get a id, we
           have to wait till the object is really created.
           TODO: a better check if we are in the portal factory!
        """
        try:
            context_id = ITG(context)
        # We must ensure that this case can't happen, any object translatable
        # will have an UUID (in any case we can be at the portal factory!)
        except TypeError:
            addAttributeTG(context, None)
            context.reindexObject(idxs=['TranslationGroup'])
            context_id = ITG(context)
        return context_id

    def query_canonical(self):
        return self.tg

    def register_translation(self, language, content):
        """ register a translation for an existing content """
        if not language and language != '':
            raise KeyError('There is no target language')

        if type(content) == str:
            content_obj = uuidToObject(content)
        else:
            content_obj = content

        # Check if exists and is not myself
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg, Language=language)
        if len(brains) > 0 and brains[0].UID != self.get_id(content_obj):
            raise KeyError("Translation already exists")

        # register the new translation in the canonical
        IMutableTG(content_obj).set(self.tg)
        content_obj.reindexObject()

        return

    def update(self):
        """ see interfaces"""
        language = ILanguage(self.context).get_language()
        self.context.reindexObject()
        # In case language is already on the translated languages we are going to orphan the old translation
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg, Language=language)
        if len(brains) == 0:
            # There is not a translation with this tg on this language
            self.register_translation(language, self.context)
        else:
            # We need to check if the language has changed
            brain = brains[0]
            content_id = self.get_id(self.context)
            if brain.UID != content_id:
                # Is a different object -> remove the old one
                # We get the old uuid
                old_object = brain.getObject()
                IMutableTG(old_object).set(NOTG)
                old_object.reindexObject()

    def add_translation(self, language):
        """ see interfaces """
        if not language and language != '':
            raise KeyError('There is no target language')
        # event
        notify(ObjectWillBeTranslatedEvent(self.context, language))
        # create the translated object
        translation_factory = ITranslationFactory(self.context)
        translated_object = translation_factory(language)
        ILanguage(translated_object).set_language(language)
        # reindex the translated object
        translated_object.reindexObject()
        # register the new translation
        self.register_translation(language, translated_object)
        # event
        notify(ObjectTranslatedEvent(self.context, \
            translated_object, language))
        return

    def remove_translation(self, language):
        """ see interfaces """
        translation = self.get_translation(language)
        IMutableTG(translation).set(NOTG)
        translation.reindexObject()

    def get_translation(self, language):
        """ see interfaces """
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg, Language=language)
        if len(brains) != 1:
            return None
        return brains[0].getObject()

    def get_translations(self):
        """ see interfaces """
        translations = {}
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg)
        for brain in brains:
            translations[brain.Language] = brain.getObject()
        return translations

    def get_restricted_translations(self):
        """ see interfaces """
        translations = {}
        brains = self.pcatalog.searchResults(TranslationGroup=self.tg,
            Language='all')
        for brain in brains:
            translations[brain.Language] = brain.getObject()
        return translations

    def get_translated_languages(self):
        """ see interfaces """
        languages = []
        brains = self.pcatalog.unrestrictedSearchResults(TranslationGroup=self.tg)
        for brain in brains:
            languages.append(brain.Language)
        return languages

    def has_translation(self, language):
        """ see interfaces """
        return language in self.get_translated_languages()



# -*- coding: utf-8 -*-
# XXX to Remove, for BBB going to be removed
from zope.interface import implements
from BTrees.OOBTree import OOBTree
from OFS.SimpleItem import SimpleItem
from plone.multilingual.interfaces import IMultilingualStorage


class CanonicalStorage(SimpleItem):

    implements(IMultilingualStorage)
    id = 'portal_multilingual'

    def __init__(self):
        self.id = id
        self.canonicals = OOBTree()

    def get_canonical(self, id):
        """ get a canonical for a specific content-id """
        canonical = None
        if id in self.canonicals:
            canonical = self.canonicals[id]
        return canonical

    def add_canonical(self, id, canonical):
        """ add a canonical
            there is a usecase where the id can already exist on the OOBTree
        """
        if not self.canonicals.insert(id, canonical):
            # We are going to remove the language on a old canonical
            # so we need to check if the canonical has other translation active
            # before removing it
            canonical_old = self.get_canonical(id)
            # check if there more than one language on the canonical
            if len(canonical_old.get_keys()) > 1:
                canonical_old.remove_item_by_id(id)
                self.remove_canonical(id)
            else:
                self.remove_canonical(id)
                del canonical_old
                # import transaction; transaction.commit()
            self.canonicals.insert(id, canonical)

    def remove_canonical(self, id):
        """ remove a canonical """
        self.canonicals.pop(id)

    def get_canonicals(self):
        """ get all canonicals """
        return self.canonicals


from zope.component import getUtility
from plone.multilingual.interfaces import IMultilingualStorage
from plone.app.uuid.utils import uuidToObject
from plone.uuid.interfaces import IUUIDGenerator
from zope.component import queryUtility
from plone.multilingual.interfaces import IMutableTG
from zope.component.hooks import getSite
from Products.CMFCore.utils import getToolByName
import transaction


def upgrade(context):
    # Move the Storage to catalog
    storage = getUtility(IMultilingualStorage)
    canonicals = storage.get_canonicals()
    already_added_canonicals = []
    generator = queryUtility(IUUIDGenerator)
    for canonical in canonicals.keys():
        canonical_object = canonicals[canonical]
        canonical_languages = canonical_object.get_keys()
        if id(canonical_object) not in already_added_canonicals:
            tg = generator()
            for canonical_language in canonical_languages:
                obj = uuidToObject(canonical_object.get_item(canonical_language))
                if obj is not None:
                    IMutableTG(obj).set(tg)
                    obj.reindexObject()
            already_added_canonicals.append(id(canonical_object))
    # Uninstall the utility
    getSite().getSiteManager().unregisterUtility(storage, IMultilingualStorage)
    del storage
    # Install the index and rebuild
    pcatalog = getToolByName(context, 'portal_catalog', None)
    if pcatalog is not None:
        indexes = pcatalog.indexes()
        if 'TranslationGroup' not in indexes:
            pcatalog.addIndex('TranslationGroup', 'FieldIndex')
            pcatalog.manage_reindexIndex(ids=['TranslationGroup'])
        else:
            pcatalog.clearFindAndRebuild()
    transaction.commit()



from zope.component import adapter
from zope.component import queryUtility

from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.lifecycleevent.interfaces import IObjectCopiedEvent

from plone.uuid.interfaces import IUUIDGenerator

from plone.multilingual.interfaces import ATTRIBUTE_NAME, ITranslatable

try:
    from Acquisition import aq_base
except ImportError:
    aq_base = lambda v: v  # soft-dependency on Zope2, fallback


@adapter(ITranslatable, IObjectCreatedEvent)
def addAttributeTG(obj, event):

    if not IObjectCopiedEvent.providedBy(event):
        if getattr(aq_base(obj), ATTRIBUTE_NAME, None):
            return  # defensive: keep existing TG on non-copy create

    generator = queryUtility(IUUIDGenerator)
    if generator is None:
        return

    tg = generator()
    if not tg:
        return

    setattr(obj, ATTRIBUTE_NAME, tg)



from setuptools import setup, find_packages
import os

version = '1.0rc1'

setup(name='plone.multilingual',
      version=version,
      description="Multilingual extensions core package",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='',
      author='Ramon Navarro Bosch',
      author_email='ramon@.nb@gmail.com',
      url='https://github.com/plone/plone.multilingual/',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['plone'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'plone.uuid',
          'plone.app.uuid'
      ],
      extras_require={
          'test': ['plone.app.testing', ],
      },
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      )




from plone.multilingual import interfaces

from zope import interface
from zope import component


@interface.implementer(interfaces.ITG)
@component.adapter(interfaces.ITranslatable)
def attributeTG(context):
    return getattr(context, interfaces.ATTRIBUTE_NAME, None)


class MutableAttributeTG(object):
    interface.implements(interfaces.IMutableTG)
    component.adapts(interfaces.ITranslatable)

    def __init__(self, context):
        self.context = context

    def get(self):
        return getattr(self.context, interfaces.ATTRIBUTE_NAME, None)

    def set(self, tg):
        tg = str(tg)
        setattr(self.context, interfaces.ATTRIBUTE_NAME, tg)

